;LCD_library.a51 *************************************************************
;* Hilfsprogramme fr die Textausgabe an das LCD-Display 
;* Vor der Verwendung initialisieren mit lcall init_disp
;* bergaberegister sind Akku und Datenpointer
;*****************************************************************************************                        
	include at89c5131.inc	; Registeradressen einbinden
 	include	LCD.inc         ; Globale Symbole definieren 

	code

; verffentlichte Unterprogramme ***************************************************************
public	init_disp, out_disp, clear_disp, clear_zeile1, clear_zeile2, clear_zeile3, clear_zeile4
public	cursor_home, cursor_rechts, cursor_links, set_cursor, cursor_on, cursor_off
public	display_shift_right, define_char, WaitWhileBusy
public	out_char, out_var, text_zeile1, text_zeile2, text_zeile3, text_zeile4
;***********************************************************************************************
;************ Display initialisieren (Anschlu am externen Bus) ***********************
init_disp:
	mov	auxr,#00000011b 	; Externer Datenbus aktivieren
					; movx erzeugt RD/ und WR/-Impulse
	mov	r7,#48
	lcall	wait			; > 15ms warten  (48*510s = 15,3ms)

	mov	r0,#WriteLCDCommand	; Adresse fr "Befehl Schreiben" in R0 laden
	mov	a,#00110000b		; Function Set: 8 Bit
	movx	@r0,a			; LCD aufwecken!

	mov	r7,#9			
	lcall	wait                    ; > 4,1ms warten (4,6ms)
	movx	@r0,a			; LCD aufwecken!
	
	mov	r7,#1
	lcall	wait 			; >100s warten  (510s)
	movx	@r0,a			; LCD aufwecken!

	; ab hier kann das Busy-Flag abgefragt werden!
	lcall	WaitWhileBusy
	mov	a,#00111000b		; Function set:
	movx	@r0,a			; 8 Bit-Mode / 2 Zeilen / 5x7 Dots
	
	lcall	WaitWhileBusy
	mov	a,#00001000b		; Display On/Off Control:
	movx	@r0,a	         	; Display OFF, Cursor OFF, Blink OFF

	lcall	WaitWhileBusy
	mov	a,#00000001b		; Display Clear
	movx	@r0,a

	lcall	WaitWhileBusy
	mov	a,#00000110b		; EntryModeSet:
	movx	@r0,a			; No shifted Display, Increment

	lcall	WaitWhileBusy
	mov	a,#00001100b		; Display On/Off Control:
	movx	@r0,a	         	; Display ON, Cursor OFF, Blink OFF
	
	ret
;	Initialisierung beendet						

;*********************************************************************************
;* Unterprogramm:       define_char
;* Datum:               Rahm, 29.11.2006
;* Beschreibung:        Definiert neue Zeichen im CGRAM (max. 8 Zeichen)
;*      Definieren der Zeichencodes:
;*               z_3:  db 00100b,01010b,10001b,10101b,10001b,01010b,00100b,00000b
;*
;*         Zeichenmatrix:
;*                 _____
;*		  |  X  |         Vor dem Unterprogrammaufruf muss die Startadresse
;*		  | X X |         der Zeichencode-Tabelle in den Datenpointer
;*		  |X   X|         und die Startadresse des neuen Zeichens
;*		  |X X X|         (im CGRAM) in den Akku geladen werden.
;*		  |X   X|            mov    dptr,#z_3
;*		  | X X |            mov    a,#010000b      ; CGRAM Adresse
;*		  |  X  |            lcall  define_char
;*		  |_____|
;*			 
;*         Zeichencode:    DDRAM        CGRAM Adresse (8 Matrixzeilen)
;*                       0000 x000    000 000 - 000 111
;*                       0000 x001    001 000 - 001 111
;*                       0000 x010    010 000 - 010 111
;*                       0000 x011    011 000 - 011 111
;*                       0000 x100    100 000 - 100 111
;*                       0000 x101    101 000 - 101 111
;*                       0000 x110    110 000 - 110 111
;*                       0000 x111    111 000 - 111 111
;*********************************************************************************
;*         Beispiel: Darstellung von z_3 an der aktuellen Cursorposition:
;*                         mov    r0,#WriteLCDData
;*                         mov    a,#02h  ; Zeichencode in Akku laden
;*                         movx   @r0,a   ; Ausgabe auf aktueller Cursorposition
;*********************************************************************************
;* Eingang:             DPTR, Akku
;* Ausgang:             ---
;* zerstrte Register:  Akku,R0,R1,R2
;*********************************************************************************
define_char:
	clr	c                       ; CGRAM-Adresse generieren
	rlc	a
	clr	c
	rlc	a
	clr	c
	rlc	a			; 
	
	orl	a,#01000000b		; CGRAM Adresse vorbereiten
	anl	a,#01111111b		; ungltiges Adressbit 7 lschen!
	push	acc
	lcall	WaitWhileBusy
	pop	acc
	mov	r0,#WriteLCDCommand	
	movx	@r0,a			; Set CGRAM address
	
	mov	r0,#WriteLCDData

next_line:
	lcall	WaitWhileBusy		; Busy-Flag und CGRAM-Adresse abfragen
	anl	a,#00000111b		; Lower 3Bit der CGRAM-Adresse ausmaskieren
	cjne	a,#00000111b,n_l1	; Wenn letzte Zeichenzeile erreicht ist, 
     	ret				;  dann Unterprogramm beenden!

n_l1:	clr	a
	movc	a,@a+dptr		; Zeichencode holen
	movx	@r0,a
	inc	dptr
	sjmp	next_line


;*********** Konstante Zeile ausgeben *********************************************
;   
;*********************************************************************************
;* Unterprogramm:       out_disp
;* Datum:               Rahm, 13.11.2006
;* Beschreibung:        Startadresse des mit /0 terminierten Ausgabestrings wird
;*                      im Datenpointer bergeben. Ausgabe ab aktueller Cursor-
;*                      position
;* Eingang:             DPTR
;* Ausgang:             ---
;* zerstrte Register:  Akku,R0
;*********************************************************************************
out_disp:
next_c:	mov	a,#0
	movc	a,@a+dptr	; Zeichen holen
	cjne	a,#0,out_c
	ret			; Rckkehr bei /0
out_c:	lcall	out_char
        inc	dptr
	sjmp	next_c

;*********** Variable Zeile ausgeben ************************************************
; Startadresse des mit /0 terminierten Ausgabestrings im internen RAM wird in r1
; bergeben. Ausgabe ab aktueller Cursorposition
out_var:
next_v:	mov	a,@r1		; Zeichen holen
	cjne	a,#0,out_v
	ret			; Rckkehr bei /0
out_v:	push	01h
	lcall	out_char
	pop	01h
        inc	r1
	sjmp	next_v

;*********** Zeichen ausgeben ********************************************
; Zeichen im Akku wird an aktueller Cursorposition ausgegeben!
out_char:
	push	acc		; Akku wird in WaitWhileBusy verndert!!
	lcall	WaitWhileBusy
	pop	acc
	mov 	r0,#WriteLCDData
	movx	@r0,a           ; Zeichen ausgeben
	ret

;*********** Display Lschen *****************************************
clear_disp:
	lcall	WaitWhileBusy
	mov	r0,#WriteLCDCommand
	mov	a,#DisplayClear
	movx	@r0,a			
	ret	

;*********** Cursor Einschalten ************************************************
cursor_on:
	lcall	WaitWhileBusy
	mov	r0,#WriteLCDCommand
	mov	a,#CursorOn
	movx	@r0,a		
	ret

;*********** Cursor Ausschalten ****************************************
cursor_off:
	lcall	WaitWhileBusy
	mov	r0,#WriteLCDCommand
	mov	a,#CursorOff
	movx	@r0,a
	ret
	
;*********** Cursor Rechtsschieben *************************************
; Cursor eine Stelle Rechts schieben.
cursor_rechts:
	lcall	WaitWhileBusy
	mov	r0,#WriteLCDCommand
	mov	a,#CursorShiftRight
	movx	@r0,a
	ret
	
;*********** Cursor Linksschieben **************************************
; Cursor eine Stelle nach Links schieben.
cursor_links:
	lcall	WaitWhileBusy
	mov	r0,#WriteLCDCommand
	mov	a,#CursorShiftLeft
	movx	@r0,a
	ret
	
;*********** Cursorhome ***********************************************
cursor_home:
	lcall	WaitWhileBusy
	mov	r0,#WriteLCDCommand
	mov	a,#ReturnHome
	movx	@r0,a		; ReturnHome
	ret

;*********** Cursorposition *******************************************
; bergabe der Cursor-Adresse im Akku
set_cursor:
	push	acc
	lcall	WaitWhileBusy
	pop	acc
	orl 	a,#10000000b	;DD RAM address set
	mov	r0,#WriteLCDCommand
	movx	@r0,a
	ret

;************ Display um eine Stelle nach rechts schieben ************
;
display_shift_right:
	lcall	WaitWhileBusy
	mov	r0,#WriteLCDCommand
	mov	a,#DisplayShiftRight
	movx	@r0,a
	ret

;**********************************************************************
loeschstring:
	$if defined (LCD20)
		db "                    ",0	; Lschstring fr 20 Zeichen/Zeile
	$elseif defined (LCD16)
		db "                ",0		; Lschstring fr 16 Zeichen/Zeile
	$endif			
;**********************************************************************
;*********** Zeilen lschen ********************************************
clear_zeile1:	
	mov	a,#Z1_Start_Adress
	sjmp	clear_zeile

clear_zeile2:
	mov	a,#Z2_Start_Adress
	sjmp	clear_zeile
	
clear_zeile3:
	mov	a,#Z3_Start_Adress
	sjmp	clear_zeile

clear_zeile4:
	mov	a,#Z4_Start_Adress

clear_zeile:
	lcall	set_cursor
	mov	dptr,#loeschstring
	lcall	out_disp
	ret

;******** Konstanten Text auf Zeile ausgeben ******************
text_zeile1:	
	mov	a,#Z1_Start_Adress
	sjmp	text_zeile

text_zeile2:
	mov	a,#Z2_Start_Adress
	sjmp	text_zeile
	
text_zeile3:
	mov	a,#Z3_Start_Adress
	sjmp	text_zeile

text_zeile4:
	mov	a,#Z4_Start_Adress

text_zeile: ; Adresse des Konstantenstrings muss im dptr stehen
	lcall	set_cursor
	lcall	out_disp
	ret
;****************************************************************************
;Warten solange das Display beschftigt ist! 
WaitWhileBusy:
	mov	r1,#ReadBusy		; Adresse fr Busy-Flag lesen in r0
busy:
	movx	a,@r1			; read busy-flag & address
	jb	busyflag,busy
	ret

;Unterprogramm: Wartezeit
wait:	mov	r6,#0ffh
wait_i:	djnz	r6,wait_i
	djnz	r7,wait
	
	ret

END














