/*===========================================================
  Funktionsbibliotheken fr LC-Display am externen Datenbus
  des 8051
  Autor: Rolf Rahm, FES
  Datum: 20.12.07
  letzte nderung: 
============================================================*/
// Headerdateien inkludieren
#include <stdio.h>
#include <5131.h>
#include <intrins.h>
#include "lcd8.h"

// Definition der LCD IO-Adressen (pdata = XRam mit Zugriff ber R0 und R1)
at 0x00 pdata char WriteLCDCommand, ReadBusy, WriteLCDData, ReadLCDData;
// WriteLCDCommand = 00h, ReadBusy = 01h, WriteLCDData = 02h, ReadLCDData = 03h

//Symboldefinitionen
#define	DisplayClear	 	0b00000001	// LCD-Befehle
#define	ReturnHome	 	0b00000010
#define	CursorOn	 	0b00001111
#define	CursorOff	 	0b00001100
#define	CursorShiftRight 	0b00010100
#define	CursorShiftLeft	 	0b00010000
#define	DisplayShiftRight 	0b00011100
#define	DisplayShiftLeft 	0b00011000

//const unsigned char AdresseZeile1 = 0x00;
//const unsigned char AdresseZeile2 = 0x40;
//const unsigned char AdresseZeile3 = 0x10; 
//const unsigned char AdresseZeile4 = 0x50;
 
//Makrodefinition
#define	WaitWhileBusy		do {;} while (ReadBusy & 0b10000000)  // Weiter wenn BF=1 !

// Globale Variablen
#if (Disp_LEN == 20)
code unsigned char loesch[] = "                    ";       // 0-terminierter Lschstring 20 Zeichen
#elif (Disp_LEN == 40)
code unsigned char loesch[] = "                                        "; // 40 Zeichen
#else
code unsigned char loesch[] = "                ";    //  16 Zeichen (default)
#endif

// private Funktionsprototypen
void init_wait(unsigned int n);		//Warteschleife nur fr Initialisierung!!
unsigned char LookUpTable(unsigned char ASCII);   //Zeicheneretzung fr Umlaute u..

// Definition der Funktionen
/*============================================================
Funktion:	InitDisp()			Rahm, 20.12.07
Beschreibung:	Initialisiert das Display im 8 Bit Modus
Eingang:	---
Ausgang:        ---
=============================================================*/
void InitDisp(void)
{
	AUXR = 0b00000011; 		// externen Datenbus aktivieren!
	init_wait(1000);
	
	WriteLCDCommand = 0b00110000;
	init_wait(1000);			// ca. 20ms warten
	WriteLCDCommand = 0b00110000;
	init_wait(500);			// ca. 10ms warten
	WriteLCDCommand = 0b00110000;
	init_wait(100);			// ca. 2ms warten

	// Ab hier Busy-Flag Abfrage mglich
	WaitWhileBusy;
	WriteLCDCommand = 0b00111000;	//Function set: 8 Bit-Mode / 2 Zeilen / 5x7 Dots
	WaitWhileBusy;
	WriteLCDCommand = 0b00001000;	//Display On/Off Control: Display OFF, Cursor OFF, Blink OFF
	WaitWhileBusy;
	WriteLCDCommand = 0b00000001;	//Display Clear
	WaitWhileBusy;
	WriteLCDCommand = 0b00000110;	//EntryModeSet: No shifted Display, Increment
	WaitWhileBusy;
	WriteLCDCommand = 0b00001100;	//Display On/Off Control: Display ON, Cursor OFF, Blink OFF
}

void init_wait(unsigned int n)
{	while(n--);
}
/*============================================================
Funktion:	DefineChar()			Rahm, 20.12.07
Beschreibung:	Erstellt ein Zeichen im Character Generator Ram
		(CGRAM)
Eingang:	CGRam Adresse unter der das Zeichen abgespeichert
		wird.
		Zeiger auf die PixelTabelle
Ausgang:        ---
==============================================================*/
void DefineChar(unsigned char *PixelTabelle, unsigned char ZeichenNr)
{
	unsigned char i;

	ZeichenNr &=0b00001111;		// Highnibble lschen
	ZeichenNr <<= 3;		// CGRam-Adresse = ZeichenNr * 8
	ZeichenNr |= 0b01000000;	// Bit 7 = 1 fr SetCGRamAdress-Kommando
	ZeichenNr &= 0b01111111;	// Bit 8 lschen (sicherheitshalber!)
	
	WaitWhileBusy;
	WriteLCDCommand = ZeichenNr;	// CGRam Adresse setzen
	
	for (i = 0; i <= 7 ; i++)	// 8 Pixelzeilen nacheinander schreiben
	{
		WaitWhileBusy;
		WriteLCDData = *(PixelTabelle+i);
	}
}


/*============================================================
Funktion:	ClearDisp()			Rahm, 20.12.07
Beschreibung:	Lscht das Display und setzt den Cursor auf 
		DDRAM-Adresse 00h
Eingang:	---
Ausgang:        ---
==============================================================*/
void ClearDisp(void)
{
	WaitWhileBusy;
	WriteLCDCommand = DisplayClear;
	WaitWhileBusy;
	WriteLCDCommand = ReturnHome;
}

/*============================================================
Funktion:	ClearZeile( n )	Rahm, 20.12.07
Beschreibung:	Lschen der Zeilen 1 bis 4 des Displays
Eingang:	Zeilennummer
Ausgang:        ---
Anmerkung:	Es wird ein Leerstring (16 Byte) auf die Zeile
		geschrieben
==============================================================*/
void ClearZeile(unsigned char Zeile)
{
	CursorAnfangZeile(Zeile);
	OutDisp(loesch);
}

//**** Cursorsteuerung **********************************
/*============================================================
Funktion:	SetCursor()			Rahm, 20.12.07
Beschreibung:	
Eingang:	DDRamAdresse
Ausgang:        ---
==============================================================*/
void SetCursor(unsigned char DDRamAdress)
{
	WaitWhileBusy;
	WriteLCDCommand = DDRamAdress | 0b10000000;	// DD-Adressoffset
}

/*============================================================
Funktion:	CursorZeile(n)			Rahm, 20.12.07
Beschreibung:	Setzt den Cursor auf den Zeilenanfang
Eingang:	Zeilennummer
Ausgang:        ---
==============================================================*/
void CursorAnfangZeile(unsigned char Zeile)
{
	switch (Zeile)
	{	case 1:	SetCursor(AdresseZeile1); break;
		case 2:	SetCursor(AdresseZeile2); break;
#if (Disp_ZEILEN == 4 )
		case 3:	SetCursor(AdresseZeile3); break;
		case 4:	SetCursor(AdresseZeile4); break;
#endif
	}
}

/*============================================================
Funktion:	CursorHome()			Rahm, 20.12.07
Beschreibung:	
Eingang:	---
Ausgang:        ---
==============================================================*/
void CursorHome(void)
{
	WaitWhileBusy;
	WriteLCDCommand = ReturnHome;
}

/*============================================================
Funktion:	CursorRechts()			Rahm, 20.12.07
Beschreibung:	
Eingang:	---
Ausgang:        ---
==============================================================*/
void CursorRechts(void)
{
	WaitWhileBusy;
	WriteLCDCommand = CursorShiftRight;
}

/*============================================================
Funktion:	CursorLinks()			Rahm, 20.12.07
Beschreibung:	
Eingang:	---
Ausgang:        ---
==============================================================*/
void CursorLinks(void)
{
	WaitWhileBusy;
	WriteLCDCommand = CursorShiftLeft;
}

/*============================================================
Funktion:	CursorEin()			Rahm, 20.12.07
Beschreibung:	
Eingang:	---
Ausgang:        ---
==============================================================*/
void CursorEin(void)
{
	WaitWhileBusy;
	WriteLCDCommand = CursorOn;
}

/*============================================================
Funktion:	CursorAus()			Rahm, 20.12.07
Beschreibung:	
Eingang:	---
Ausgang:        ---
==============================================================*/
void CursorAus(void)
{
	WaitWhileBusy;
	WriteLCDCommand = CursorOff;
}


//******** Zeichenausgabe *************************************
/*============================================================
Funktion:	OutDisp()			Rahm, 20.12.07
Beschreibung:	Gibt einen /0-terminierten String ab der 
		aktuellen Cursorposition aus
Eingang:	Zeiger auf String
Ausgang:        ---
==============================================================*/
void OutDisp(unsigned char *pChar)
{
	unsigned char i;
	
	for (i=0;*(pChar+i) != '\0';i++)
	{	
		OutChar(*(pChar+i));
	}
}
/*============================================================
Funktion:	OutChar()			Rahm, 20.12.07
Beschreibung:	Gibt ein Zeichen auf der aktuellen Cursor-
		position aus.
Eingang:	Zeiger auf Zeichen
Ausgang:        ---
==============================================================*/
void OutChar(unsigned char Zeichen)
{
	Zeichen = LookUpTable(Zeichen);
	WaitWhileBusy;
	WriteLCDData = Zeichen;
}

/*============================================================
Funktion:	TextZeile(*text, n)		Rahm, 20.12.07
Beschreibung:	Gibt einen mit /0-terminierten String auf
		Zeile 1 ... 4 aus
Eingang:	Zeiger auf String, Zeilennummer
Ausgang:        ---
==============================================================*/
void TextZeile(unsigned char *pChar, unsigned char Zeile)
{
	CursorAnfangZeile(Zeile);
	OutDisp(pChar);
}

/*============================================================
Funktion:	OutHex(n)		Rahm, 3.6.09
Beschreibung:	Gibt das Byte n als 2 stelligen Hex-Wert aufs
		Display
Eingang:	Byte
Ausgang:        ---
==============================================================*/
void OutHex(unsigned char Dual)
{
	unsigned char buf;
	
        buf = Dual & 0xF0;  	//Highnibble
        buf = _swapnibble_(buf);
        if (buf <= 9)  buf += '0';     
        else           buf += 0x37;
	OutChar(buf);
	
	buf = Dual & 0x0F;	//Lownibble
	if (buf <= 9)  buf += '0';
	else           buf += 0x37;
	OutChar(buf);
	
}

/*============================================================
Funktion:	OutDez(n)		Rahm, 3.6.09
Beschreibung:	Gibt das Byte n als 3 stelligen Dez-Wert aufs
		Display. Fhrende Nullen werden zu blank.
Eingang:	Byte
Ausgang:        ---
==============================================================*/
void OutDez(unsigned char Dual)
{
	unsigned char buf;
	bit flag;
	
	buf = Dual / 100;		//Hunderter
	if (buf != 0) OutChar(buf+'0');
	else 
	 { OutChar(' ');
	   flag = 0;
	 }
	
	Dual = Dual % 100;		//Zehner
	buf = Dual / 10;                       
	
	if (buf != 0) OutChar(buf+'0');
	else if (!flag) OutChar(' ');
	else OutChar(buf+'0');

	buf = Dual % 10;		//Einer
	OutChar(buf+'0');
	
}

/*============================================================
Funktion:	LookUpTable(char)		Rahm, 4.6.09
Beschreibung:	Ersetzt ein ASCII-Umlaut durch den ent-
		sprechenden Display-Code.
Eingang:		ASCII-Code
Ausgang:    Display-Code
==============================================================*/
unsigned char LookUpTable(unsigned char ASCII)
{
	switch (ASCII)
	{
	  case '': return 0xDF;
	  case '': return 0xE1;
         case '': return 0xEF;
         case '': return 0xF5;
         case '': return 0xE1;
         case '': return 0xEF;
         case '': return 0xF5;
         case '': return 0xE2;
         case '': return 0xE4;
         case '\\': return 0xA4;
         case '': return 0xD3;
        
    }
}
//============================================================


unsigned char ReadBackChar(unsigned char DDRAMAdress)
{	
		WaitWhileBusy;
		return ReadLCDData;
}
//------------------------------------------------------------
unsigned char ReadBackCursor(void)
{
 		WaitWhileBusy;
 		return (ReadBusy & 0b01111111);
}
/*============================================================
Funktion:	DisplayLesen(StartAdresse, EndAdresse)	Rahm, 4.5.2010
Beschreibung:	Liest den Displayinhalt zwischen Start- und
		Endadresse ein!
Eingang:	Pointer auf Pufferstring,Display StartAdresse;
		Display EndAdresse
Ausgang:      ---
==============================================================*/
void DisplayLesen(unsigned char *buf,unsigned char StartAdresse,unsigned char EndAdresse)
{
	unsigned char i;
	
	SetCursor(StartAdresse);   //Cursor auf Startadresse setzen
	for (i = 0; i < (EndAdresse-StartAdresse); i++)
	{
		WaitWhileBusy;
		*(buf+i) = ReadLCDData;
	}
	*(buf+i) = '\0';
}


