;*********************************************************************************
;* Hilfsdatei zur Ansteuerung einer Tastenfeldmatrix
;*********************************************************************************
;* Diese Hilfsprogramme sind geschrieben fr den Anschluss einer Tastenfeldmatrix
;* an den externen Datenbus des Controllers.
;* Wichtig: Der externe Speicherzugriff muss erst per Softwarebefehl aktiviert
;*          werden! Folgende Zeile muss vor dem 1. movx-Befehl ausgefhrt werden!
;*	mov	auxr,#00000011b 	; Externer Datenbus aktivieren
;*					; movx erzeugt RD/ und WR/-Impulse
;*					; ALE-Signale werden nur bei movx-Befehlen
;*					; erzeugt!
;*********************************************************************************
;* Aufruf der Unterprogramme:
;*--------------------------------------------------------------------------------
;*	lcall	get_first_keycode	; Fragt alle Tasten der Reihe ab!
;*					; Akku = Keycode der 1. bettigten Taste!
;*					; Akku = 0 --> Keine Taste war besttigt!
;*
;*	mov	a,#keycode		; Liefert den Ascii-Code einer Taste
;*	lcall   get_key_ascii		; Akku = Ascii-Zeichen des Keycodes!
;*					;          (Definiert in Key_tab)
;*	
;*	mov	a,#keycode		; Abfrage einer einzigen Taste
;*	lcall	scan_key		; Carry-Flag = 0 --> Taste nicht bettigt!
;*					; Carry-Flag = 1 --> Taste bettigt!
;*
;*	lcall	wait_on_key		; Wartet bis beliebige Taste bettigt!
;*					; Akku = Keycode der bettigten Taste!
;*
;*	mov	a,#keycode		; berprft Taste auf Ziffer 0...9
;*	lcall   ist_ziffer		; Akku = Ziffer oder Ascii-Zeichen
;*					; Carry-Flag = 0 --> Ziffer
;*					; Carry-Flag = 1 --> Keine Ziffer
;*********************************************************************************
;* Aufbau Tastaturmatrix 4x4: (Tastenbelegung anpassen!!!)
;* 	Beispiel fr Zuordnung der Keycodes zu den ASCII-Zeichen der Tastatur
;*
;*                             Spalten|Zeilen
;*	Anschlussbelegung:     D C B A 4 3 2 1
;*	                       | | | | | | | |
;*
;*		  Spalten:     4    3    2    1        Zeilenadressen fr 
;*                            ---  ---  ---  ---	 movx-Befehl:
;*		   Zeile: A  | 7 || 8 || 9 || > |	   04h
;*  		 Keycode:     ==4  ==3  ==2  ==1         
;*		   Zeile: B  | 4 || 5 || 6 || < |	   05h
;*   	         Keycode:     ==8  ==7  ==6  ==5        
;*		   Zeile: C  | 1 || 2 || 3 ||   |	   06h
;*               Keycode:     ==12 ==11 ==10 ==9
;*		   Zeile: D  | C || 0 || E || . |	   07h
;*               Keycode:     --16 --15 --14 --13
;*
;*********************************************************************************

	include c51rd2.inc
	include tastatur.inc
	
	public	get_key_ascii, scan_key, wait_on_key, get_first_keycode, ist_ziffer
	
	code
;**********************************************************************************
;* Die Tasteaturtabellen mssen an das tatschliche Tastaturlayout angepasst werden
;* Zum finden der richtigen Tastenzuordnung kann das Programm tastatur_test.a51
;* verwendet werden!
;* Die Symbole Spalten4...Spalten1 werden in der Datei Tastatur.inc definiert.
;********************************************************************************** 
$if defined (Spalten4)
key_tab:	;Keycode 1   2   3   4
	       	db	7Eh,'9','8','7'    ; Zeile A
	       	;Keycode 5   6   7   8     
	       	db	7Fh,'6','5','4'    ; Zeile B
	       	;Keycode 9   10  11  12     
	       	db	' ','3','2','1'    ; Zeile C
	       	;Keycode 13  14  15  16
		db	'.','E','0','C'    ; Zeile D	       	

$endif
$if defined (Spalten3)
key_tab:	;Keycode 1   2   3   
		db	'.','0','#'        ; Zeile A
	       	;Keycode 4   5   6         
	       	db	'1','2','3'        ; Zeile B
	       	;Keycode 7   8   9          
	       	db	'4','5','6'        ; Zeile C
	       	;Keycode 10  11  12     
	       	db	'7','8','9'        ; Zeile D	
$endif
$if defined (Spalten2)
key_tab:	;Keycode 1   2       
		db	'0','1'            ; Zeile A
	       	;Keycode 3   4             
	       	db	'2','3'            ; Zeile B
	       	;Keycode 5   6              
	       	db	'4','5'            ; Zeile C
	       	;Keycode 7   8          
	       	db	'6','7'            ; Zeile D
$endif
$if defined (Spalten1)
key_tab:	;Keycode 1           
		db	'1'                ; Zeile A
	       	;Keycode 2                 
	       	db	'2'                ; Zeile B
	       	;Keycode 3                  
	       	db	'3'                ; Zeile C
	       	;Keycode 4              
	       	db	'4'                ; Zeile D
$endif 
	
	Zeile	equ	R0	; Zeilenadresse muss R0 oder R1 sein!!
	Spalte	equ	R1
	Keycode	equ	R2
	Zeichen	equ	r3

;*********************************************************************************
;* Unterprogramm: 	get_first_keycode
;* Datum:		Rahm, 1.11.2006
;* Beschreibung:        Scannt alle Tastenfelder durch und liefert den
;*                      Keycode der ersten bettigten Taste.
;*                      Ist keine Taste bettigt wird 0 zurckgegeben!
;* Eingang:             --
;* Ausgang:             Akku
;* zerstrte Register:  R0,R1,R2
;*********************************************************************************
get_first_keycode:
	mov	Keycode,#AnzTasten  ; Keycode = #AnzahlTasten (z.B. 16 bei 4x4)
	mov	Zeile,#ZeileD+1  ; Zeile = Zeile4_Adresse + 1
next_Zeile:
	dec	Zeile		; Zeile = Zeile - 1
	movx	a,@Zeile	; Tastaturzeile einlesen,
	cpl	a		; Akku = [x,x,x,x,D3,D2,D1,D0]
	anl	a,#00001111b	; Akku = [0,0,0,0,D3,D2,D1,D0]
	swap	a               ; Akku = [D3,D2,D1,D0,0,0,0,0]
	mov	Spalte,#AnzSpalten	
				; Spaltenzhler = #AnzahlSpalten

	clr	c		; Carry-Flag lschen
next_Spalte:
	rlc	a		; Prfbit in Carryflag schieben
	jc	break		; Taste bettigt? Dann Prfung erfolgreich!
				;    Sonst
	dec	Keycode		;     Keycode = Keycode - 1
	djnz	Spalte,next_Spalte
				;     Spaltenzhler = Spaltenzhler-1
				;     Wenn letzte Spalte erreicht, dann nchste Zeile
				;     einlesen!
	cjne	Zeile,#ZeileA,next_Zeile 
				; Wenn Zeile <> Zeile1, dann nchste Zeile scannen!
break:				;     sonst, fertig
	mov	a,Keycode	; Akku = Keycode

	ret

;get_first_keycode:
	mov	Keycode,#AnzTasten-1

next_k:	mov	a,Keycode
	inc	a
	lcall	scan_key
	jc	betaetigt
	djnz	Keycode,next_k
	mov	a,Keycode
	ret
betaetigt:
	mov	a,Keycode
	inc	a
	ret


;*********************************************************************************
;* Unterprogramm: 	get_key_ascii
;* Datum:		Rahm, 1.11.2006
;* Beschreibung:        Erhlt einen Keycode im Akku. 
;*			Holt das entsprechende Ascii-Zeichen aus der 
;*			Keycode-Tabelle und gibt es im Akku zurck!
;*                      
;* Eingang:             Akku
;* Ausgang:             Akku
;* zerstrte Register:  ---
;*********************************************************************************
get_key_ascii:
	push	dph
	push	dpl
	mov	dptr,#key_tab
	dec	a
	movc	a,@a+dptr
	pop	dpl
	pop	dph
	ret
	
;*********************************************************************************
;* Unterprogramm: 	scan_key
;* Datum:		Rahm, 1.11.2006
;* Beschreibung:        Scannt die Taste deren Keycode im Akku bergeben wird.
;*			Liefert 1, wenn Sie bettigt ist
;*                      und 0, wenn Sie unbettigt ist!
;*                      
;* Eingang:             Akku = Keycode (1..16)
;* Ausgang:             Carry-Flag (1: bettigt; 0: nicht bettigt)
;* zerstrte Register:  B, R0, Akku, R2
;*********************************************************************************
scan_key:
	mov	b,#AnzSpalten	; Anzahl Spalten in B
	dec	a       	; Akku = Keycode - 1 (0..15)
	div	ab		; a = Zeilenindex (0..3); b = Spaltenindex (0..3)
	add	a,#ZeileA	; Adressoffset
	mov	Zeile,a		; Zeile = ZeileA + Zeilenindex
	movx	a,@Zeile	; Zeile einlesen
	cpl	a		; 0 = Taste bettigt ==> invertieren
	anl	a,#00001111b    ; Highnibble ist undefiniert
	mov	Spalte,b	; Spalte = Spaltenindex (1..4)
	inc	Spalte
w_sp:	rrc	a 		; Akkubit (r0) ins Carryflag schieben!
	djnz	Spalte,w_sp
	
	ret
	
;*********************************************************************************
;* Unterprogramm: 	wait_on_key
;* Datum:		Rahm, 1.11.2006
;* Beschreibung:        Wartet bis eine beliebige Taste bettigt wird!
;*			Gibt den Keycode der entsprechenden Taste im Akku zurck!
;*                      
;* Eingang:             ---
;* Ausgang:             Akku
;* zerstrte Register:  B, R0, Akku
;*********************************************************************************
wait_on_key:
	lcall	get_first_keycode	; [Akku] = Keycode
	jz	wait_on_key		; Wenn Keycode = 0, dann weiter scannen!
	lcall	entprell
	push	acc
released:
	lcall	get_first_keycode
	jnz	released
;	lcall	scan_key
;	jc	released
	lcall	entprell
	pop	acc
	ret

;*********************************************************************************
;* Unterprogramm: 	ist_ziffer
;* Datum:		Rahm, 23.12.2006
;* Beschreibung:        Bekommt den Keycode eines Zeichens
;*			
;*                      
;* Eingang:             Akku = Keycode
;* Ausgang:             Akku 		A <- Ziffer ; A <- Ascii-Zeichen
;*			Carry-Flag	0 = Ziffer ; 1 = Keine Ziffer
;* zerstrte Register:  Akku, C
;*********************************************************************************

ist_ziffer:
	lcall	get_key_ascii
	mov	Zeichen,a		; Zeichen = Ascii(Keycode)
	
	mov	a,#39h
	clr	c
	subb	a,Zeichen		; Asciicode > 39h
	jc	no_num			;   dann, keine Ziffer!

	mov	a,Zeichen
	subb	a,#30h			; Asciicode < 30h
	jc	no_num			;   dann, keine Ziffer!
	
	sjmp	exit_ziffer

no_num:	mov	a,Zeichen	

exit_ziffer:
	ret


;**********************************************************************************
;
entprell:
	mov	r0,#0
e1:	djnz	r0,e1
	ret

end		

