/*===========================================================
  Funktionsbibliotheken fr Tastatur am externen Datenbus
  des 8051
  Autor: Rolf Rahm, FES
  Datum: 2.6.09
  letzte nderung: 7.6.09
============================================================*/
// Headerdateien inkludieren
//#include <stdio.h>
#include <5131.h>
#include "tastatur8.h"
#include <intrins.h>
#include "lcd8.h"
//#include <stdlib.h>

// Definition der Tastatur-Zeilenadressen (pdata = XRam mit Zugriff ber R0 und R1)
at 0x04 pdata char ZeileA,ZeileB,ZeileC,ZeileD;
// ZeileA = 04h, ZeileB = 05h, ZeileC = 06h, ZeileD = 07h

bdata	unsigned char Zeile;
sbit	KB0 = Zeile^0;
sbit	KB1 = Zeile^1;
sbit	KB2 = Zeile^2;
sbit	KB3 = Zeile^3;

// Globale Variablen
#define SPALTEN 4
#define ZEILEN 4
#define TAB 3
#define _INPUT_PROMPT_       // Schalter aktivieren fr Eingabeprompt
				// Dann muss das Display initialisiert sein!!

// Tastaturlayout:
//      Keycode <-> ASCII-Zeichen

const unsigned char keytab[] = { 0,0x7E,'9','8','7',
                                   0x7F,'6','5','4',
                                   ' ' ,'3','2','1',
                                   '.' ,'E','0','C'
                               };

// private Funktionsprototypen

// Definition der Funktionen
/*============================================================
Funktion:	get_keycode()		Rahm, 30.05.09
Beschreibung:	Scannt alle Tastenfelder und liefert den Keycode
                der ersten bettigeten Taste! Ist keine Taste 
                bettigt, wird 0 zurckgegeben!
Eingang:	---
Ausgang:        keycode
=============================================================*/
unsigned char GetKeycode(void)
{
	unsigned char z;
	
	for(z = 0; z!=4; z++)
	{ Zeile = ~(*(&ZeileA+z));
	  Zeile &= 0b00001111;
	  if (KB0) return z*4 +1;
	  else if (KB1) return z*4 +2;
	  else if (KB2) return z*4 +3;
	  else if (KB3) return z*4 +4;
	}
	return 0;	
}

/*============================================================
Funktion:	KeyToAscii()		Rahm, 6.06.09
Beschreibung:	Ermittelt den Ascii-Code eines Keycodes
Eingang:	---
Ausgang:        keycode
=============================================================*/
unsigned char KeyToAscii(unsigned char keycode)
{
	return keytab[keycode];
}

/*============================================================
Funktion:	AsciiToKey()		Rahm, 6.06.09
Beschreibung:	Liefert den Keycode eines Ascii-Zeichens.
                Ist das Zeichen nicht im Tastaturlayout (keytab)
                definiert, wird 0 zurckgegeben!
Eingang:	Ascii-Code
Ausgang:        keycode oder 0
=============================================================*/
unsigned char AsciiToKey(unsigned char ASCII)
{
	unsigned char keycode;
	
	for (keycode = 1; keycode != 17; keycode++)
	  if (ASCII == keytab[keycode])
	    return keycode;
	
	return 0;
}
/*============================================================
Funktion:	ScanKey(keycode)	Rahm, 6.06.09
Beschreibung:	Scannt eine einzelne Taste 
Eingang:	keycode
Ausgang:        nicht bettigt:  bit = 0
		bettigt:        bit = 1
=============================================================*/
bit ScanKey(unsigned char keycode)
{
	unsigned char zeil,spalt;
	
	keycode--;
	
	zeil = keycode / SPALTEN;
	spalt = keycode % SPALTEN;
	
	Zeile = *(&ZeileA + zeil);
	Zeile = ~Zeile;
	Zeile &= 0b00001111;
	
	switch (spalt)
	{	case 0: return _testbit_(KB0);
		case 1: return _testbit_(KB1);
		case 2: return _testbit_(KB2);
		case 3: return _testbit_(KB3);
	}
}

/*============================================================
Funktion:	WaitOnKey()	 Rahm, 6.06.09
Beschreibung:	Wartet bis eine bestimmte Taste gedrckt ist! 
Eingang:	---
Ausgang:        keycode
=============================================================*/
unsigned char WaitOnKey(void)
{
	unsigned char keycode;
	while(!(keycode = GetKeycode()));
	return keycode;
}
#ifdef _INPUT_PROMPT_
/*============================================================
Funktion:	EingabePrompt(Puffer, StartAdresse, EndAdresse)	Rahm, 4.5.2010
Beschreibung:	Stellt auf dem Display einen Eingabeprompt fr
		Integer-Zahlen zur Verfgung. Der Puffer enthlt
		den Eingabestring fr Zahleneingaben
Eingang:	Pointer auf Pufferstring, Display StartAdresse;
		Display EndAdresse
Ausgang:      Anzahl eigelesener Zeichen + '\0'
==============================================================*/
unsigned char EingabePrompt(unsigned char *buf, unsigned char StartAdresse,unsigned char AnzahlZeichen)
{
	unsigned char taste;
	unsigned char i = 0;

	SetCursor(StartAdresse);
	CursorEin();
	
	while((taste = GetKeycode()) != AsciiToKey('E'))   // Solange bis ENT gedrckt
	{
		switch (taste = KeyToAscii(taste))
	    	{
	    		case '0':
	    		case '1':
	    		case '2':
	    		case '3':
	    		case '4':
	    		case '5':
	    		case '6':
	    		case '7':
	    		case '8':
	    		case '9':
	 			if (i<AnzahlZeichen)
	 			{ OutChar(taste);
	 			  i++;
	 			}
		       	while(GetKeycode());  //warten bis taste losgelassen
	    			break;
			case '\x7F':
				if (i >= 1)
				{ CursorLinks();
				  OutChar(' ');
				  CursorLinks();
  				  i--;
				}
		       	while(GetKeycode());  //warten bis taste losgelassen				
				break;
/*			case '\x7E':
				if (i < AnzahlZeichen)
				{ 
				  if (ReadBackChar(ReadBackCursor()) != ' ')
				  { CursorRechts();
				    i++;
				  }
				  CursorLinks();
				}
		       	while(GetKeycode() != 0);  //warten bis taste losgelassen				
				break;
*/
	    		case 'C':
	    			SetCursor(StartAdresse);
	    			for(;i>0;i--)
	    				OutChar(' ');
	    			SetCursor(StartAdresse);
	    			break;
	    			
	    		default:
	    			break;
	    	}	    	
	}
      	while(GetKeycode());  //warten bis taste losgelassen
	CursorAus();
	DisplayLesen(buf,StartAdresse,StartAdresse + i);
	
	return i;
}
#endif
